local _, L = ...;
local VERSION = 11326;
local playerName = UnitName("player");
function WorldBuffTracker_GetVersion()
  return VERSION;
end
local wbt_faction = "Alliance";
local wbtString = "[WorldBuffTracker] ";
local WorldBuffTrackerFrame = CreateFrame("Frame");
local addonInitialized = 99999999999999;
local WorldBuffTrackerEvents = {}
WorldBuffTracker = {
  ["PlaySound"] = true,
  ["SelfWhisper"] = true,
  ["VersionControl"] = true,
  ["LastSync"] = 0,
  ["1"] = {
    ["Timestamp"] = 0,
    ["Yell"] = 0,
    ["Version"] = VERSION,
    ["Sender"] = playerName,
    ["Origin"] = playerName,
  },
  ["2"] = {
    ["Timestamp"] = 0,
    ["Yell"] = 0,
    ["Version"] = VERSION,
    ["Sender"] = playerName,
    ["Origin"] = playerName,
  },
  ["3"] = {
    ["Timestamp"] = 0,
    ["Yell"] = 0,
    ["Version"] = VERSION,
    ["Sender"] = playerName,
    ["Origin"] = playerName,
  },
}

WorldBuffTrackerZones = {
  ["Horde"] = {
    [1411] = true, [1454] = true, [1413] = true;
  },
  ["Alliance"] = {
      [1453] = true, [1429] = true;
    }
}

WorldBuffTrackerTTB = {
  ["1"] = 13, ["2"] = 17, ["3"] = 17,
}

WorldBuffTrackerBuffCDs = {
  ["1"] = 10800, ["2"] = 21600, ["3"] = 28800,
  ["SF"] = 1500;
}

function WorldBuffTracker_Initialize()
  if(GetSpellBookItemName(1, "spell") ~= nil) then
    if(WorldBuffTrackerSpells[GetLocale()] ~= nil) then
      WorldBuffTrackerSpells = WorldBuffTrackerSpells[GetLocale()];
      WorldBuffTrackerNPCs = WorldBuffTrackerNPCs[GetLocale()];
    else
      WorldBuffTrackerSpells = WorldBuffTrackerSpells["enUS"];
      WorldBuffTrackerNPCs = WorldBuffTrackerNPCs["enUS"];
    end
    wbt_faction, _ = UnitFactionGroup("player");
    if(WorldBuffTracker["LastSync"] - GetServerTime() < -300) then
      C_ChatInfo.SendAddonMessage("WBT-0", VERSION, "GUILD");
      WorldBuffTracker["LastSync"] = GetServerTime();
    end
    addonInitialized = GetTime() + 0.5;
    WorldBuffTrackerFrame:SetScript("OnUpdate", WorldBuffTracker_DelayTracking);
    WorldBuffTracker_VerifyTable();
    WorldBuffTracker_TimeTableTest();
  end
end

function WorldBuffTracker_DelayTracking()
  if(addonInitialized - GetTime() < 0) then
    addonInitialized = 99999999999999;
    WorldBuffTrackerFrame:SetScript("OnUpdate", nil);
    for k, v in pairs(WorldBuffTrackerEvents) do
      WorldBuffTrackerFrame:RegisterEvent(k);
    end
    WorldBuffTrackerFrame:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
    WorldBuffTrackerFrame:UnregisterEvent("CHAT_MSG_MONSTER_YELL");
    WorldBuffTrackerEvents:ZONE_CHANGED_NEW_AREA();
  end
end

function WorldBuffTrackerEvents:COMBAT_LOG_EVENT_UNFILTERED(...)
  local _, eventType, _,
      _, sourceName, _, _,
      destGUID, destName, _, _,
      spellId, spellName, _, auraType, _ = CombatLogGetCurrentEventInfo();
    if((eventType == "SPELL_AURA_APPLIED" or eventType == "SPELL_AURA_REFRESHED")
      and destName == playerName and auraType == "BUFF" and WorldBuffTrackerSpells[spellName] ~= nil) then
        local buffID = WorldBuffTrackerSpells[spellName];
        if(buffID == 2) then
          if(WorldBuffTracker["2"]["Yell"] == WorldBuffTracker["3"]["Yell"]) then
            return
          else
            buffID = (WorldBuffTracker["2"]["Yell"] > WorldBuffTracker["3"]["Yell"]) and "2" or "3";
          end
        end
        if(WorldBuffTracker_BuffIsValid(spellName, buffID)) then
          if(IsInGuild()) then
            C_ChatInfo.SendAddonMessage("WBT-" .. buffID, VERSION, "GUILD");
          else
            WorldBuffTracker_HandleBuffById(buffID);
          end
        end
    end
end

function WorldBuffTrackerEvents:ZONE_CHANGED_NEW_AREA(...)
  if(WorldBuffTrackerZones[wbt_faction][C_Map.GetBestMapForUnit("player")] ~= nil) then
    WorldBuffTrackerFrame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
    WorldBuffTrackerFrame:RegisterEvent("CHAT_MSG_MONSTER_YELL");
  else
    WorldBuffTrackerFrame:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
    WorldBuffTrackerFrame:UnregisterEvent("CHAT_MSG_MONSTER_YELL");
  end
end

function WorldBuffTrackerEvents:CHAT_MSG_MONSTER_YELL(msg, npc, ...)
  if(WorldBuffTrackerNPCs[wbt_faction][npc] == nil) then
    return;
  end
  npc = tostring(WorldBuffTrackerNPCs[wbt_faction][npc]);
  WorldBuffTracker_TimeTableTest();
  if(WorldBuffTracker[npc]["Timestamp"] == 0) then
    local t = GetServerTime();
    if(WorldBuffTracker[npc]["Yell"] < t) then
      if(npc ~= "1" or (strfind(msg, L["ThrallYell"]) ~= nil)) then
        C_ChatInfo.SendAddonMessage("WBT-A", npc .. "_" .. VERSION, "GUILD");
        WorldBuffTracker_BuffAnnounce(npc, t);
      end
    end
  end
end

function WorldBuffTrackerEvents:CHAT_MSG_ADDON(prefix, message, channel, sender, ...)
  local sender, _ = strsplit("-", sender);
  local t = GetServerTime();
  if(prefix == "WBT-A" and channel == "GUILD") then
    local buff, version = strsplit("_", message);
    version = WorldBuffTracker_GetVersionFromString(version);
    if(WorldBuffTracker[buff] ~= nil and version >= 11326) then
      WorldBuffTracker_BuffAnnounce(buff, t);
    end
  elseif(prefix == "WBT-1") then
    WorldBuffTracker_HandleWarchief(sender, WorldBuffTracker_GetVersionFromString(message), t);
  elseif(prefix == "WBT-2") then
    WorldBuffTracker_HandleOnyxia(sender, WorldBuffTracker_GetVersionFromString(message), t);
  elseif(prefix == "WBT-3") then
    WorldBuffTracker_HandleNefarian(sender, WorldBuffTracker_GetVersionFromString(message), t);
  elseif(prefix == "WBT-0") then
    if(channel == "GUILD" and sender ~= playerName) then
      WorldBuffTracker_SendSyncData(sender);
    elseif(channel == "WHISPER") then
      WorldBuffTracker_HandleSync(message, sender);
    end
  end
end

function WorldBuffTracker_BuffAnnounce(buffID, t)
    if(WorldBuffTracker[buffID]["Yell"] < t) then
      WorldBuffTracker[buffID]["Yell"] = t + 3600;
      local buffString;
      for k,v in pairs(WorldBuffTrackerSpells) do
        if(tostring(v) == buffID or (v == 2 and buffID == "3")) then
          buffString = k;
          break;
        end
      end
      local buffInfo = "";
      if(buffID ~= "1") then
        buffInfo = buffID == "2" and " (Onyxia) " or " (Nefabian) ";
      end
      print(wbtString .. buffString .. buffInfo .. " incoming!");
      WorldBuffTracker_AlertPlayer(buffString, true, true);
    end
end

local function WorldBuffTrackerFrame_Command(msg, editbox)
  if(strlower(msg) == "version") then
    if(WorldBuffTracker["VersionControl"]) then
      print(wbtString .. L["Version announce: disabled."])
    else
      print(wbtString .. L["Version announce: enabled."])
    end
    WorldBuffTracker["VersionControl"] = not WorldBuffTracker["VersionControl"];
    return;
  end
  WorldBuffTracker_TimeTableTest();
  local t = GetServerTime();
  print("--- WorldBuffTracker ---");
  local buffs = {};
  for k, v in pairs(WorldBuffTrackerSpells) do
    buffs[v] = k;
  end
  for v=1,3 do
    local k = buffs[((v > 2) and 2 or v)];
    local buffSource = "";
    if(v == 2) then
      buffSource = " (" .. L["Onyxia"] .. ")";
    elseif(v == 3) then
      buffSource = " (" .. L["Nefarian"] .. ")";
    end
    print(k .. ":" .. buffSource);
    v = tostring(v);
    local bT = WorldBuffTracker[v]["Timestamp"];
      if(bT > 0) then
        local cd = WorldBuffTrackerBuffCDs[v];
        local sources = " ";
        if(WorldBuffTracker[v]["Origin"] ~= WorldBuffTracker[v]["Sender"]) then
          sources = sources .. "(" .. WorldBuffTracker[v]["Origin"] .. "→" .. WorldBuffTracker[v]["Sender"] .. ")";
        else
          sources = sources .. "(" .. WorldBuffTracker[v]["Sender"] .. ")";
        end
        print("    " .. string.format(L["Popped: %s Next: %s in: %s"], date('%H:%M:%S', bT), date('%H:%M:%S', bT + cd), WorldBuffTracker_TimeUntilDate(bT - t + cd))
          .. sources);
      else
        print("    " .. L["No data."]);
      end
    end
  print("------------------------")
end

function WorldBuffTracker_TimeUntilDate(seconds)
  local str = "";
  if(seconds > 3600) then
    local hours = math.floor(seconds / 3600);
    str = hours .. "h ";
    seconds = seconds - 3600 * hours;
  end
  if(seconds > 60) then
    local minutes = math.floor(seconds / 60);
    str = str .. minutes .. "m ";
    seconds = seconds - 60 * minutes;
  end
  if(seconds > 0) then
    str = str .. seconds .. "s";
  end
  return str;
end

function WorldBuffTracker_AlertPlayer(buff, playSound, whisper)
  if(playSound) then
    PlaySoundFile("Interface/AddOns/WorldBuffTracker/resources/sound/beware.ogg", "Master");
  end
  if(whisper) then
    SendChatMessage(wbtString .. buff .. " incoming!", "WHISPER", nil, UnitName("player"));
  end
end

SLASH_WORLDBUFFS1, SLASH_WORLDBUFFS2, SLASH_WORLDBUFFS3 = '/worldbuffs', '/worldbufftracker', '/wbt'

SlashCmdList["WORLDBUFFS"] = WorldBuffTrackerFrame_Command;

WorldBuffTrackerFrame:SetScript("OnUpdate", WorldBuffTracker_Initialize);
WorldBuffTrackerFrame:SetScript("OnEvent", function(self, event, ...)
  WorldBuffTrackerEvents[event](self, ...);
end);
C_ChatInfo.RegisterAddonMessagePrefix("WBT-0");
C_ChatInfo.RegisterAddonMessagePrefix("WBT-1");
C_ChatInfo.RegisterAddonMessagePrefix("WBT-2");
C_ChatInfo.RegisterAddonMessagePrefix("WBT-3");
C_ChatInfo.RegisterAddonMessagePrefix("WBT-A");
